from president_speech.db.parquet_interpreter import get_parquet_full_path
import pandas as pd
import typer

# 추가 요청사항
# 검색 단어가 등장하는 ROW 에 speech_text 속에 해당 단어가 나오는 횟수를 모두 count 하여 그 총합을 대통령별로 다시 sum
def add_keyword_count(df: pd.DataFrame, keyword: str) -> pd.DataFrame:
    """
    DataFrame에 keyword_count 컬럼을 추가하여 반환합니다.
    각 speech_text에서 keyword가 등장하는 횟수를 계산합니다.
    """
    # keyword_count 컬럼 추가
    df['keyword_count'] = df['speech_text'].str.count(keyword)
    return df

def group_by_count(keyword: str, asc: bool=False, rcnt: int=12, keyword_sum: bool=False) -> pd.DataFrame:
    data_path = get_parquet_full_path() # 데이터 경로 가져오기 
    df = pd.read_parquet(data_path)  # 데이터 프레임으로 로드
    fdf = df[df['speech_text'].str.contains(keyword, case=False)] # 키워드가 들어간 연설 필터링 
    if(keyword_sum):
        fdf = add_keyword_count(fdf.copy(), keyword)
        gdf = fdf.groupby("president").agg(
            count=("speech_text", "size"),  # 연설 개수
            keyword_sum=("keyword_count", "sum")  # keyword 발생 횟수 합산
        )
        sdf = gdf.sort_values(by=["keyword_sum", "count"], ascending=[asc, asc]).reset_index()
    else:
        gdf = fdf.groupby("president").size().reset_index(name="count")
        sdf = gdf.sort_values(by='count', ascending=asc).reset_index(drop=True)
    rdf = sdf.head(rcnt)
    return rdf

def print_group_by_count(keyword: str, asc: bool=False, rcnt: int=12, keyword_sum: bool=False):
    df = group_by_count(keyword, asc, rcnt, keyword_sum)
    print(df.to_string(index=False))

def entry_point():
    typer.run(print_group_by_count)



from eda_ji.cli import group_by_count
import pandas as pd
import pytest
    
presidents_speeches = {
    "박정희": 513,
    "이승만": 438,
    "노태우": 399,
    "김대중": 305,
    "문재인": 275,
    "김영삼": 274,
    "이명박": 262,
    "전두환": 242,
    "노무현": 230,
    "박근혜": 111,
    "최규하": 14,
    "윤보선": 1
}

def test_search():
    row_count = 13
    # When
    df = group_by_count(keyword="자유", asc=True, rcnt=row_count)
    # assert
    assert isinstance(df, pd.DataFrame) 
    assert len(df) < row_count

def test_noascen():
    row_count = 3
    is_asc = True
    # When
    df = group_by_count(keyword="자유", asc=is_asc, rcnt=row_count)
    # assert
    assert isinstance(df, pd.DataFrame)
    assert df.iloc[0]["president"] == "윤보선"
    assert len(df) == row_count

def test_ascen():
    row_count = 3
    is_asc = False
    # When
    df = group_by_count(keyword="자유", asc=is_asc, rcnt=row_count)
    # assert
    assert isinstance(df, pd.DataFrame)
    assert df.iloc[0]["president"] == "박정희"
    assert len(df) == row_count

@pytest.mark.parametrize("is_asc, president", [(True,"윤보선"),(False,"박정희")])
def test_sort(is_asc,president):
    row_count = 3

    # When
    df = group_by_count(keyword="자유", asc=is_asc, rcnt=row_count)
    
    # assert
    assert isinstance(df, pd.DataFrame)
    assert df.iloc[0]["president"] == president
    assert len(df) == row_count

def test_all_count():
    # When
    df = group_by_count("자유") 
    presidents_speeches = {}
    for i in range(len(df)):
        presidents_speeches[df.iloc[i]["president"]] = df.iloc[i]["count"]
        #assert president_row.iloc[i]["count"] == s_count
    # assert
    assert presidents_speeches["박정희"] == 513 
    assert presidents_speeches["이승만"] == 438 
    assert presidents_speeches["노태우"] == 399 
    assert presidents_speeches["김대중"] == 305 
    assert presidents_speeches["문재인"] == 275
    assert presidents_speeches["김영삼"] == 274 
    assert presidents_speeches["이명박"] == 262 
    assert presidents_speeches["전두환"] == 242 
    assert presidents_speeches["노무현"] == 230 
    assert presidents_speeches["박근혜"] == 111 
    assert presidents_speeches["최규하"] == 14 
    assert presidents_speeches["윤보선"] == 1 
    

def test_all_count_keyword_sum():
    # given
    # global dict
    
    # when
    df = group_by_count("자유", keyword_sum = True)
    
    # assert
    assert isinstance(df, pd.DataFrame)
    assert "keyword_sum" in df.columns
    # count 보다 keyword_sum이 크거나 같음을 확인 assert
    # 1 - 모범 case 
    assert all (df["count"] <= df["keyword_sum"])  # 모든 값이 다 True인지 알려주는 내장함수 all(데이터 프레임 요소별로 비교함)
    # 2
    for row in df.itertuples():
        assert row.keyword_sum >= row.count
    # 3!!!
    for i in range(len(df)):
        keyword_sum = df.iloc[i, df.columns.get_loc("keyword_sum")]
        count = df.iloc[i, df.columns.get_loc("count")]
        assert keyword_sum >= count


